module osd_generator ( clk, ena, hs, vs, osd_ena_out, osd_image,
				p8ena_disp, p8ena_font, p8adr, p8dat );

input  clk, ena, hs, vs ;

output osd_ena_out ;
reg    osd_ena_out ;
output [2:0] osd_image ;

input p8ena_disp, p8ena_font;
input [15:0] p8adr;
input [7:0] p8dat;

reg   [9:0] disp_x,dly1_disp_x,dly2_disp_x;
reg   [8:0] disp_y,dly1_disp_y,dly2_disp_y;

reg   dena,dly1_dena,dly2_dena,dly3_dena,dly4_dena;
reg   [7:0] dly1_letter, dly2_letter;

wire [12:0] font_pos;
wire [8:0]  disp_pos;
wire [2:0]  osd_image;
// ****************************************************************************************************************************
// LPM_RAM_DP
// ****************************************************************************************************************************
wire [7:0] sub_wire0;
wire [7:0] letter = sub_wire0[7:0];
// w access = 'b1100000a aaaaaaaa
//altsyncram	altsyncram_component_osd_mem ( .wren_a ( p8ena_disp ), .clock0 (clk), .clock1 (clk), .clocken1 (ena),
//											.address_a (p8adr[8:0]), .address_b (disp_pos[8:0]),
//											.data_a (p8dat[7:0]), .q_b (sub_wire0));

	altdpram	altdpram_component (

				.inclock    (clk),
				.inclocken  (1),
				.wren       (p8ena_disp),
				.wraddress  (p8adr[8:0]),
				.data       (p8dat[7:0]),

				.outclock   (clk),
				.outclocken (ena),
				.rdaddress  (disp_pos[8:0]),
				.q          (sub_wire0) );

	defparam
		altdpram_component.indata_aclr = "OFF",
		altdpram_component.indata_reg = "INCLOCK",
		altdpram_component.intended_device_family = "Cyclone II",
		altdpram_component.lpm_type = "altdpram",
		altdpram_component.outdata_aclr = "OFF",
		altdpram_component.outdata_reg = "OUTCLOCK",
		altdpram_component.rdaddress_aclr = "OFF",
		altdpram_component.rdaddress_reg = "OUTCLOCK",
		altdpram_component.rdcontrol_aclr = "OFF",
		altdpram_component.rdcontrol_reg = "UNREGISTERED",
		altdpram_component.use_eab = "ON",
		altdpram_component.width = 8,
		altdpram_component.widthad = 9,
		altdpram_component.width_byteena = 1,
		altdpram_component.wraddress_aclr = "OFF",
		altdpram_component.wraddress_reg = "INCLOCK",
		altdpram_component.wrcontrol_aclr = "OFF",
		altdpram_component.wrcontrol_reg = "INCLOCK";

/*
	defparam
		altsyncram_component_osd_mem.intended_device_family = "Cyclone",
		altsyncram_component_osd_mem.operation_mode = "DUAL_PORT",
		altsyncram_component_osd_mem.width_a = 8,
		altsyncram_component_osd_mem.widthad_a = 9,
		altsyncram_component_osd_mem.width_b = 8,
		altsyncram_component_osd_mem.widthad_b = 9,
		altsyncram_component_osd_mem.lpm_type = "altsyncram",
		altsyncram_component_osd_mem.width_byteena_a = 1,
		altsyncram_component_osd_mem.outdata_reg_b = "CLOCK1",
		altsyncram_component_osd_mem.indata_aclr_a = "NONE",
		altsyncram_component_osd_mem.wrcontrol_aclr_a = "NONE",
		altsyncram_component_osd_mem.address_aclr_a = "NONE",
		altsyncram_component_osd_mem.address_reg_b = "CLOCK1",
		altsyncram_component_osd_mem.address_aclr_b = "NONE",
		altsyncram_component_osd_mem.outdata_aclr_b = "NONE",
		altsyncram_component_osd_mem.ram_block_type = "AUTO",
		altsyncram_component_osd_mem.init_file = "osd_mem.mif";
*/
// ****************************************************************************************************************************
wire [1:0] sub_wire1;
wire [1:0] osd_img = sub_wire1[1:0];
// w access = 'b111aaaaa aaaaaaaa
altsyncram	altsyncram_component_osd_font ( .wren_a ( p8ena_font ), .clock0 (clk), .clock1 (clk), .clocken1 (ena),
											.address_a (p8adr[12:0]), .address_b (font_pos[12:0]),
											.data_a (p8dat[1:0]), .q_b (sub_wire1));
	defparam
		altsyncram_component_osd_font.intended_device_family = "Cyclone",
		altsyncram_component_osd_font.operation_mode = "DUAL_PORT",
		altsyncram_component_osd_font.width_a = 2,
		altsyncram_component_osd_font.widthad_a = 13,
		altsyncram_component_osd_font.width_b = 2,
		altsyncram_component_osd_font.widthad_b = 13,
		altsyncram_component_osd_font.lpm_type = "altsyncram",
		altsyncram_component_osd_font.width_byteena_a = 1,
		altsyncram_component_osd_font.outdata_reg_b = "CLOCK1",
		altsyncram_component_osd_font.indata_aclr_a = "NONE",
		altsyncram_component_osd_font.wrcontrol_aclr_a = "NONE",
		altsyncram_component_osd_font.address_aclr_a = "NONE",
		altsyncram_component_osd_font.address_reg_b = "CLOCK1",
		altsyncram_component_osd_font.address_aclr_b = "NONE",
		altsyncram_component_osd_font.outdata_aclr_b = "NONE",
		altsyncram_component_osd_font.ram_block_type = "AUTO",
		altsyncram_component_osd_font.init_file = "osd_font.mif";
// ****************************************************************************************************************************
// ****************************************************************************************************************************



assign disp_pos[4:0]  = disp_x[8:4] ;
assign disp_pos[8:5]  = disp_y[7:4] ;

assign font_pos[12:6] = letter[6:0] ;
assign font_pos[2:0]  = dly2_disp_x[3:1] ;
assign font_pos[5:3]  = dly2_disp_y[3:1] ;

assign osd_image[1:0] = osd_img[1:0];
assign osd_image[2]   = dly2_letter[7];

always @ ( posedge clk ) begin

if (ena) begin

dly1_disp_x <= disp_x;
dly2_disp_x <= dly1_disp_x;

dly1_disp_y <= disp_y;
dly2_disp_y <= dly1_disp_y;

dly1_letter <= letter;
dly2_letter <= dly1_letter;

dly1_dena   <= dena;
dly2_dena   <= dly1_dena;
dly3_dena   <= dly2_dena;
dly4_dena   <= dly3_dena;

osd_ena_out  <= dly2_dena; // need to delay by 4


	if (disp_x[9] || disp_y[8]) dena <= 0;
	else dena <= 1;

	 if (vs) disp_y[8:0] <= 9'b111111111;
else if (hs) begin
					disp_x[9:0] <= 10'b0000000000; //10'b0000000000;
					if (!disp_y[8] | (disp_y[8:7] == 2'b11)) disp_y <= disp_y + 1;
				end else if (!disp_x[9]) disp_x <= disp_x + 1;


end // ena
end // always@clk
endmodule
